/***************************************************************************
 *                                                                         *
 *   gtrm.cpp       (begin: Jun 25 2004)                                   *
 *                                                                         *
 *   Parallel IQPNNI - Important Quartet Puzzle with NNI                   *
 *                                                                         *
 *   Copyright (C) 2003-2004 by Le Sy Vinh, Arndt von Haeseler             *
 *   {vinh}@cs.uni-duesseldorf.de                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gtrm.h"

#include "opturtree.h"

#include "brent.h"

/* compute 1 PAM rate matrix, its eigensystem, and the inverse matrix thereof */

//=========================================================================
//=========================================================================
void GTRM::createRateParameters () {
	/*
	double sumGenPam_ = tsAG_ + tsCT_ + tvAC_ + tvAT_ + tvCG_ + tvGT_;
	tsAG_ /=  sumGenPam_;
	tsCT_ /=  sumGenPam_;
	tvAC_ /=  sumGenPam_;
	tvAT_ /=  sumGenPam_;
	tvCG_ /=  sumGenPam_;
	tvGT_ /=  sumGenPam_;
	*/
	tvGT_ = 0.1;
	unitRateMat_[BASE_A][BASE_C] = tvAC_;
	unitRateMat_[BASE_C][BASE_A] = tvAC_;

	unitRateMat_[BASE_A][BASE_G] = tsAG_;
	unitRateMat_[BASE_G][BASE_A] = tsAG_;

	unitRateMat_[BASE_A][BASE_T] = tvAT_;
	unitRateMat_[BASE_T][BASE_A] = tvAT_;

	unitRateMat_[BASE_C][BASE_G] = tvCG_;
	unitRateMat_[BASE_G][BASE_C] = tvCG_;

	unitRateMat_[BASE_C][BASE_T] = tsCT_;
	unitRateMat_[BASE_T][BASE_C] = tsCT_;

	unitRateMat_[BASE_G][BASE_T] = tvGT_;
	unitRateMat_[BASE_T][BASE_G] = tvGT_;

	unitRateMat_[BASE_A][BASE_A] = 0.0;
	unitRateMat_[BASE_C][BASE_C] = 0.0;
	unitRateMat_[BASE_G][BASE_G] = 0.0;
	unitRateMat_[BASE_T][BASE_T] = 0.0;
}

//=========================================================================
//=========================================================================
void GTRM::init () {
	createRateParameters ();
	tranprobmat(nState_);
} /* tranprobmat */

void GTRM::reInit () {
	createRateParameters ();
	tranprobmat(nState_);
}

//----------------------------------------------------------------------------------------------------------------------------------------
double GTRM::cmpNegLogLi (double value) {
	if (isActPam_ == TS_AG)
		tsAG_ = value;
	if (isActPam_ == TS_CT)
		tsCT_ = value;
	if (isActPam_ == TV_AC)
		tvAC_ = value;
	if (isActPam_ == TV_AT)
		tvAT_ = value;
	if (isActPam_ == TV_CG)
		tvCG_ = value;
	if (isActPam_ == TV_GT)
		tvGT_ = value;

	double oriTsAG_, oriTsCT_, oriTvAC_, oriTvAT_, oriTvCG_, oriTvGT_;
	oriTsAG_ = tsAG_;
	oriTsCT_ = tsCT_;
	oriTvAC_ = tvAC_;
	oriTvAT_ = tvAT_;
	oriTvCG_ = tvCG_;
	oriTvGT_ = tvGT_;

	reInit ();

	tsAG_ = oriTsAG_;
	tsCT_ = oriTsCT_;
	tvAC_ = oriTvAC_;
	tvAT_ = oriTvAT_;
	tvCG_ = oriTvCG_;
	tvGT_ = oriTvGT_;


	opt_urtree.cmpLiNd ();
	//opt_urtree.opt (MAX_IT_UR_BR_PAM_OPT);
	double logLi_ = opt_urtree.getLogLi ();
	return -logLi_;
}


//--------------------------------------------------------------------
//optimize the tsTvRatio basing Brent method
bool GTRM::optPam () {
	double fx_, error_;
	double oriTsAG_ = tsAG_;
	double oriTsCT_ = tsCT_;
	double oriTvAC_ = tvAC_;
	double oriTvAT_ = tvAT_;
	double oriTvCG_ = tvCG_;
	double oriTvGT_ = tvGT_;


	if (isMasterProc())
		std::cout <<"Optimizing general reversible model parameters ..." << endl;
	std::cout.precision (5);

	Brent::turnOnOptedPam ();
	double logLi_ = opt_urtree.cmpLogLi ();
	//  std::cout <<"Log likelihood: " << logLi_ << endl;


	isActPam_ = TV_AC;
	if (isMasterProc()) {
		cout << "    1. Transversion rate from A to C = ";
		cout.flush();
	}
	tvAC_= optOneDim (MIN_GEN_PAM, tvAC_, MAX_GEN_PAM,
	                  EPS_MODEL_PAM_ERROR, &fx_, &error_);
	if (isMasterProc())
		cout << tvAC_ << endl;

	reInit ();

	//------------------------------/------------------------------/------------------------------/------------------------------/------------------------------/------------------------------
	isActPam_ = TS_AG;
	if (isMasterProc()) {
		cout << "    2. Transition   rate from A to G = ";
		cout.flush();
	}
	tsAG_= optOneDim (MIN_GEN_PAM, tsAG_, MAX_GEN_PAM,
	                  EPS_MODEL_PAM_ERROR, &fx_, &error_);
	if (isMasterProc())
		cout << tsAG_ << endl;
	reInit ();

	//------------------------------/------------------------------/------------------------------/------------------------------/------------------------------/------------------------------
	isActPam_ = TV_AT;
	if (isMasterProc()) {
		cout << "    3. Transversion rate from A to T = ";
		cout.flush();
	}
	tvAT_= optOneDim (MIN_GEN_PAM, tvAT_, MAX_GEN_PAM,
	                  EPS_MODEL_PAM_ERROR, &fx_, &error_);
	if (isMasterProc())
		cout << tvAT_ << endl;
	reInit ();

	//------------------------------/------------------------------/------------------------------/------------------------------/------------------------------/------------------------------
	isActPam_ = TV_CG;
	if (isMasterProc()) {
		cout << "    4. Transversion rate from C to G = ";
		cout.flush();
	}
	tvCG_= optOneDim (MIN_GEN_PAM, tvCG_, MAX_GEN_PAM,
	                  EPS_MODEL_PAM_ERROR, &fx_, &error_);
	if (isMasterProc())
		cout << tvCG_ << endl;
	reInit ();

	//------------------------------/------------------------------/------------------------------/------------------------------/------------------------------/------------------------------

	isActPam_ = TS_CT;

	if (isMasterProc()) {
		cout << "    5. Transition   rate from C to T = ";
		cout.flush();
	}
	tsCT_= optOneDim (MIN_GEN_PAM, tsCT_, MAX_GEN_PAM,
	                  EPS_MODEL_PAM_ERROR, &fx_, &error_);
	if (isMasterProc())
		cout << tsCT_ << endl;
	if (isMasterProc())
		cout << "    6. Transversion rate from G to T =  " << tvGT_  << endl;

	reInit ();

	opt_urtree.cmpLiNd ();
	logLi_ = opt_urtree.getLogLi ();

	Brent::turnOffOptedPam ();

	// copy parameters to mymodel
	//mymodel.
	return ( fabs (tsAG_ - oriTsAG_) > EPS_MODEL_PAM_ERROR ||
	         fabs (tsCT_ - oriTsCT_) > EPS_MODEL_PAM_ERROR ||
	         fabs (tvAC_ - oriTvAC_) > EPS_MODEL_PAM_ERROR ||
	         fabs (tvAT_ - oriTvAT_) > EPS_MODEL_PAM_ERROR ||
	         fabs (tvCG_ - oriTvCG_) > EPS_MODEL_PAM_ERROR ||
	         fabs (tvGT_ - oriTvGT_) > EPS_MODEL_PAM_ERROR  );
}


