/***************************************************************************
 *                                                                         *
 *                  (begin: Feb 20 2003)                                   *
 *                                                                         *
 *   Parallel IQPNNI - Important Quartet Puzzle with NNI                   *
 *                                                                         *
 *   Copyright (C) 2005 by Le Sy Vinh, Bui Quang Minh, Arndt von Haeseler  *
 *   Copyright (C) 2003-2004 by Le Sy Vinh, Arndt von Haeseler             *
 *   {vinh,minh}@cs.uni-duesseldorf.de                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// lind.h: interface for the LiNd class.
//
//////////////////////////////////////////////////////////////////////

#ifndef LIND_H
#define LIND_H

#include "innd.h"
#include "vec.h"
#include "cube.h"



/*
this class contain all information information of likelihood at this node
*/
class LiNd {
public:
	//the constructor function
	LiNd ();

	/*liCube_[ptnNo][rateNo][baseNo] is the likelihood at
	this node coressponding to ptnNo, rateNo, baseNo
	*/
	Cube liNdCube_;

	int isEx_;
	int id_;

	//parNdNo_ of this likelihood node
	int parNdNo_;

	//parNdNo_ of this likelihood node
	int parBrNo_;

	int isExDesNdCreated_;
	Vec<int> exDesNdNoLs_;

	int isCmped_;

	void writeInf ();

	//set the  id, ndType, parrent node, and branch no for this node
	void set (int id, EX_IN ndType, int parNdNo, int parBrNo);

	int getId ();

	Vec<int> &getExDesNd ();

	void turnOffIsCmped ();

	void setParNdBr (int newParNdNo, int newParBrNo);

	//return the likelihood of these arguments
	LDOUBLE cmpLi (int ptnNo, int rateNo, int baseNo);


	//--------------------------------------------------------------------
	//return the likelihood of these argument
	inline LDOUBLE getLiNd (int ptnNo, int rateNo, int baseNo) {
		return liNdCube_[ptnNo][rateNo][baseNo];
	}

	//get the parrent node, and branch no for this node
	void getPar (int &parNdNo, int &parBrNo);

	//compute the likelihood for all ptnNo, rateNo, baseNo at this node
	void cmpLi ();

	//compute the likelihood for all ptnNo, rateNo, baseNo at this node
	void cmpLiUniformRate ();

	//compute the likelihood for all ptnNo, rateNo, baseNo at this node
	void cmpLiSpecificRate ();

	//compute the likelihood for all ptnNo, rateNo, baseNo at this node
	void cmpLiGammaRate ();

	//compute the likelihood for all ptnNo, rateNo, baseNo at this node, for codon-based model
	void cmpLiSelection ();
	
	//reCompute the likelihood for all ptnNo, rateNo, baseNo of this node
	void reCmpLi ();


	//create the external descendant nodes no list of this node
	void createExDesNd (Vec<int> &exDesNdNoLs);

	//create the external descendant nodes no list of this node
	void createExDesNd ();

	//set memory for this class
	void setLimit ();

	//release the memory of this class
	void release ();

	//the destructor function
	virtual ~LiNd ();

	/*
protected:
	int work_start;
	int work_end;
	LDOUBLE *local_vec;
	LDOUBLE *global_vec;
	*/
}
;// end of class

#endif //LIINND
